<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2010 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "include/config.php";
include_once "include/common.php";
include_once "include/site.php";
include_once "include/orderby.php";
include_once "include/stemming.php";
include_once "include/search.php";

function display_search_operator($item_name, $value) {
  global $strings;

  print "<select name=\"$item_name\">\n";
  print emit_option("or", $strings['search_union'], $value == "or");
  print emit_option("and", $strings['search_intersect'], $value == "and");
  print "</select>\n";
}

function display_advanced_search_entry($database, $search_data, $search_operator, $masters_only) {
  global $thm_elem;
  global $strings;
  global $search_interface;

  theme_display_navigator_box_top($search_interface['text_advanced']['idx'], $search_interface, "100%");

  print "<table class=\"search\">";

  print "<tr><th align=\"right\">".$strings['photo_identifier']."</th><td >";
  print "<input type=\"text\" name=\"identifier\" value=\"".$search_data['identifier']."\" size=\"50\" /></td><td>";
  display_search_operator("identifier_operator", $search_operator['identifier']);
  print "</td></tr>";

  print "<tr><th align=\"right\">".$strings['generic_title']."</th><td >";
  print "<input type=\"text\" name=\"title\" value=\"".$search_data['title']."\" size=\"50\" /></td><td>";
  display_search_operator("title_operator", $search_operator['title']);
  print "</td></tr>";

  print "<tr><th align=\"right\">".$strings['generic_author']."</th><td >";
  print "<input type=\"text\" name=\"author\" value=\"".$search_data['author']."\" size=\"50\" /></td><td>";
  display_search_operator("author_operator", $search_operator['author']);
  print "</td></tr>";

  print "<tr><th align=\"right\">".$strings['generic_keywords']."</th><td >";
  print "<input type=\"text\" name=\"keyword\" value=\"".$search_data['keyword']."\" size=\"50\" /></td><td>";
  display_search_operator("keyword_operator", $search_operator['keyword']);
  print "</td></tr>";

  print "<tr><th align=\"right\">".$strings['generic_caption']."</th><td >";
  print "<input type=\"text\" name=\"caption\" value=\"".$search_data['caption']."\" size=\"50\" /></td><td>";
  display_search_operator("caption_operator", $search_operator['caption']);
  print "</td></tr>";

  print "<tr><th align=\"right\">".$strings['generic_location']."</th><td >";
  print "<input type=\"text\" name=\"location\" value=\"".$search_data['location']."\" size=\"50\" /></td><td>";
  display_search_operator("location_operator", $search_operator['location']);
  print "</td></tr>";

  print "<tr><th align=\"right\">".$strings['search_date_range']."</th><td >";
  print emit_date_input_form("start_date", $search_data['start_date'], 'sql');
  print "</td><td>";

  display_search_operator("date_operator", $search_operator['date']);

  print "</td></tr>";
  print "<tr><th align=\"right\"></th><td >";
  print emit_date_input_form("end_date", $search_data['end_date'], 'sql');
  print "</td><td>".emit_date_field_combo("date_type", $search_data['date_type'])."</td></tr>";

  if ($masters_only)
    $checked = ' checked="checked" ';
  else
    $checked = '';
  print "<tr><td align=\"center\" colspan=\"2\"><input type=\"checkbox\" name=\"only_masters\" $checked />".$strings['search_masters_only'] ."</td></tr>";
  print "<tr><td align=\"center\" colspan=\"2\">";
  print $thm_elem['button.search'];
  print $thm_elem['button.clear'];
  print "</td></tr>";
  print "</table>";
  
  theme_display_navigator_box_bottom("100%");
}

$froms = array();
$fields = array();

$search_data['identifier'] = isset($_REQUEST['identifier']) ? validate_search_string($_REQUEST['identifier']) : "";
$search_data['title']      = isset($_REQUEST['title']) ? validate_search_string($_REQUEST['title']) : "";
$search_data['author']     = isset($_REQUEST['author']) ? validate_search_string($_REQUEST['author']) : "";
$search_data['keyword']    = isset($_REQUEST['keyword']) ? validate_search_string($_REQUEST['keyword']) : "";
$search_data['caption']    = isset($_REQUEST['caption']) ? validate_search_string($_REQUEST['caption']) : "";
$search_data['location']   = isset($_REQUEST['location']) ?  validate_search_string($_REQUEST['location']) : "";
$search_data['start_date']   = isset($_REQUEST['start_date']) ?  validate_search_string($_REQUEST['start_date']) : "";
$search_data['end_date']   = isset($_REQUEST['end_date']) ?  validate_search_string($_REQUEST['end_date']) : "";
$search_data['date_type']   = isset($_REQUEST['date_type']) ?  validate_search_string($_REQUEST['date_type']) : "";

$search_operator['identifier'] = isset($_REQUEST['identifier_operator']) ? $_REQUEST['identifier_operator'] : "";
$search_operator['title']      = isset($_REQUEST['title_operator']) ? $_REQUEST['title_operator'] : "";
$search_operator['author']     = isset($_REQUEST['author_operator']) ? $_REQUEST['author_operator'] : "";
$search_operator['keyword']    = isset($_REQUEST['keyword_operator']) ? $_REQUEST['keyword_operator'] : "";
$search_operator['caption']    = isset($_REQUEST['caption_operator']) ? $_REQUEST['caption_operator'] : "";
$search_operator['location']   = isset($_REQUEST['location_operator']) ? $_REQUEST['location_operator'] : "";
$search_operator['date'] = isset($_REQUEST['date_operator']) ? $_REQUEST['date_operator'] : "";

if ($search_data['start_date'] == 'null')  $search_data['start_date'] = '';
if ($search_data['end_date'] == 'null')  $search_data['end_date'] = '';

$master = isset($_REQUEST['only_masters']);

$offset = pg_escape_string(isset($_REQUEST['offset']) ? $_REQUEST['offset'] : 0);

$order_by = $po_options['order_by'];

$database      = site_prolog();

site_header($strings['generic_search']);
site_navigator(2);

$keywords['identifier'] = extract_keywords($search_data['identifier'], $po_options['search_enable_stemming']);
$keywords['title']      = extract_keywords($search_data['title'], $po_options['search_enable_stemming']);
$keywords['author']     = extract_keywords($search_data['author'], $po_options['search_enable_stemming']);
$keywords['keyword']    = extract_keywords($search_data['keyword'], $po_options['search_enable_stemming']);
$keywords['caption']    = extract_keywords($search_data['caption'], $po_options['search_enable_stemming']);
$keywords['location']   = extract_keywords($search_data['location'], $po_options['search_enable_stemming']);

if (!$search_data['identifier'] &&
    !$search_data['title'] &&
    !$search_data['author'] &&
    !$search_data['keyword'] &&
    !$search_data['caption'] &&
    !$search_data['start_date'] &&
    !$search_data['end_date'] &&
    !$search_data['location']) {

  if (!$master && ($po_options['search_masters_only'] != 'f')) {
    $master = TRUE;
  }  
  
  site_navigator_status($strings['search_string'], "");
  print "<form action=\"search.text.advanced.php\" method=\"post\" accept-charset=\"".$strings['formats_encoding']."\">";
  display_advanced_search_entry($database, $search_data, $search_operator, $master);
  print "</form>";
  site_footer($database);
  site_epilog($database);
  exit();
}

$status_string=$strings['search_displaying'];

$sql_search_string['identifier'] = build_sql_search_string($keywords['identifier'], "photo.identifier");
$sql_search_string['title']      = build_sql_search_string($keywords['title'],      "photo.title");
$sql_search_string['author']     = build_sql_search_string($keywords['author'],     "photo.author");
$sql_search_string['keyword']    = build_sql_search_string_keywords($keywords['keyword']);

$sql_search_string['caption']    = build_sql_search_string($keywords['caption'],    "photo.caption");
$sql_search_string['location']   = build_sql_search_string($keywords['location'],   "(view_location.place|||view_location.city|||view_location.state|||view_location.country)");

if (isset($_REQUEST['start_date']) && 
    isset($_REQUEST['end_date']) &&
    isset($_REQUEST['date_type']) &&
    $_REQUEST['start_date'] != 'null' &&
    $_REQUEST['start_date'] != '' &&
    $_REQUEST['end_date'] != 'null' &&
    $_REQUEST['end_date'] != '') {
  $start_date = pg_escape_string($_REQUEST['start_date']);
  $end_date = pg_escape_string($_REQUEST['end_date']);
  $date_type = pg_escape_string($_REQUEST['date_type']);
  $sql_search_string['date'] = "$date_type BETWEEN '$start_date' AND '$end_date'";
 } else {
  $sql_search_string['date'] = "";
 }

$sql_combined_search_string = "";

if ($sql_search_string['identifier'] != "") {
  if ($sql_combined_search_string != "")
    $sql_combined_search_string = $sql_combined_search_string ." ".$search_operator['identifier']." " .$sql_search_string['identifier'];
  else
    $sql_combined_search_string = $sql_search_string['identifier'];
}

if ($sql_search_string['title'] != "") {
  if ($sql_combined_search_string != "")
    $sql_combined_search_string = $sql_combined_search_string ." ".$search_operator['title']." " .$sql_search_string['title'];
  else
    $sql_combined_search_string = $sql_search_string['title'];
}

if ($sql_search_string['author'] != "") {
  if ($sql_combined_search_string != "")
    $sql_combined_search_string = $sql_combined_search_string ." ".$search_operator['author']." " .$sql_search_string['author'];
  else
    $sql_combined_search_string = $sql_search_string['author'];
}

if ($sql_search_string['keyword'] != "") {
  if ($sql_combined_search_string != "")
    $sql_combined_search_string = $sql_combined_search_string ." ".$search_operator['keyword']." " .$sql_search_string['keyword'];
  else
    $sql_combined_search_string = $sql_search_string['keyword'];
}

if ($sql_search_string['caption'] != "") {
  if ($sql_combined_search_string != "")
    $sql_combined_search_string = $sql_combined_search_string ." ".$search_operator['caption']." " .$sql_search_string['caption'];
  else
    $sql_combined_search_string = $sql_search_string['caption'];
}

if ($sql_search_string['location'] != "") {
  if ($sql_combined_search_string != "")
    $sql_combined_search_string = $sql_combined_search_string ." ".$search_operator['location']." " .$sql_search_string['location'];
  else
    $sql_combined_search_string = $sql_search_string['location'];
}

if ($sql_search_string['date'] != "") {
  if ($sql_combined_search_string != "")
    $sql_combined_search_string = $sql_combined_search_string ." ".$search_operator['date']." " .$sql_search_string['date'];
  else
    $sql_combined_search_string = $sql_search_string['date'];
}

/* this is just to avoid having three separate queries */
if ($sql_combined_search_string == "") {
  $sql_combined_search_string = "photo.identifier=null";
}

if (!$offset) {
  $offset = 0;
}

$photo_view = $po_options['photo_view'];

if ($photo_view == "slide") {
  $limit = $po_options['photos_per_page'];
 } else {
  $limit = $po_options['rows_list_view'];
 }

if ($master)
  $master = "and photo_version.master = 't'";
 else
  $master = "";

$filter = " ($sql_combined_search_string)
            $master";

if (sizeof($keywords['location'])) {
    $froms[] .= "view_location";
    $filter .=  " and view_location.identifier = photo.location ";
  }

$search_result = get_photo_query($database, $fields, $froms, $filter, 0, 0, $order_by);
$num_of_matches = sizeof($search_result);

/* Sanity checks */
if ($limit > $num_of_matches) {
  $limit = $num_of_matches;
 }
while ($num_of_matches && $offset >= $num_of_matches) {
  $offset -= $limit;
 }
if ($offset < 0) {
  $offset = 0;
 }
if (($offset + $limit) > $num_of_matches) {
  $items = $num_of_matches - $offset;
 } else {
  $items = $limit;
 }

$itemized_search_string['identifier'] = build_itemized_search_string($search_data['identifier'], "identifier");
$itemized_search_string['title']      = build_itemized_search_string($search_data['title'],      "title");
$itemized_search_string['author']     = build_itemized_search_string($search_data['author'],     "author");
$itemized_search_string['keyword']    = build_itemized_search_string($search_data['keyword'],    "keyword");
$itemized_search_string['caption']    = build_itemized_search_string($search_data['caption'],    "caption");
$itemized_search_string['location']   = build_itemized_search_string($search_data['location'],   "location");

if ($num_of_matches == 0) {
  site_navigator_status($strings['search_no_matches']." $itemized_search_string", "");
  print "<form action=\"search.text.advanced.php\" method=\"post\" accept-charset=\"".$strings['formats_encoding']."\">";
  display_advanced_search_entry($database, $search_data, $search_operator, $master);
  print "</form>";
  site_footer($database);
  site_epilog($database);
  exit();
}

site_navigator_status($strings['search_searched_for']." $itemized_search_string",  $strings['search_displaying'] ." ". display_photo_index_status($offset, $limit, $num_of_matches));
print "<form action=\"search.text.advanced.php\" method=\"post\" accept-charset=\"".$strings['formats_encoding']."\">";
display_advanced_search_entry($database, $search_data, $search_operator, $master);
print "</form>";
print "&nbsp;<br/>\n";

$arguments = "";

if ($master) {
  $arguments .= "&amp;only_masters=true";
 }

$search_string =
"identifier=".$search_data['identifier'].
"&amp;title=".$search_data['title'].
"&amp;author=".$search_data['author'].
"&amp;keyword=".$search_data['keyword'].
"&amp;caption=".$search_data['caption'].
"&amp;location=".$search_data['location'].
"&amp;start_date=".$search_data['start_date'].
"&amp;end_date=".$search_data['end_date'].
"&amp;date_type=".$search_data['date_type'].
"&amp;date_operator=".$search_operator['date'].
"&amp;dentifier_operator=".$search_operator['identifier'].
"&amp;title_operator=".$search_operator['title'].
"&amp;author_operator=".$search_operator['author'].
"&amp;keyword_operator=".$search_operator['keyword'].
"&amp;caption_operator=".$search_operator['caption'].
"&amp;location_operator=".$search_operator['location'];

$search_string_exp = str_replace(" ", "+", $search_string);

print "<form  name=\"photoList\" method=\"post\" action=\"folder.content.php\" accept-charset=\"".$strings['formats_encoding']."\">";
print "<input type=\"hidden\" name=\"return\" value=\"search.text.advanced.php?$search_string_exp$search_string$arguments\"/>";

display_photo_navigator("search.text.advanced.php?$search_string_exp$arguments", $offset, $limit, $num_of_matches);

if ($photo_view == "slide") {
  $checkboxes = display_photo_slides("search_advanced", FALSE, $search_result, $offset, $items, true, true); 
 } else {
  $checkboxes = display_photo_list("search_advanced", FALSE, $search_result, $offset, $items);
 }
display_photo_navigator("search.text.advanced.php?$search_string_exp$arguments", $offset, $limit, $num_of_matches);

print "&nbsp;<br/>\n";

$itemized_search_string['identifier'] = build_itemized_search_string($search_data['identifier'], "identifier");
$itemized_search_string['title']      = build_itemized_search_string($search_data['title'],      "title");
$itemized_search_string['author']     = build_itemized_search_string($search_data['author'],     "author");
$itemized_search_string['keyword']    = build_itemized_search_string($search_data['keyword'],    "keyword");
$itemized_search_string['caption']    = build_itemized_search_string($search_data['caption'],    "caption");
$itemized_search_string['location']   = build_itemized_search_string($search_data['location'],   "location");

site_navigator_status($strings['search_searched_for']." $itemized_search_string",  $strings['search_displaying'] ." ". display_photo_index_status($offset, $limit, $num_of_matches));

print "<div class=\"actions\">";

if ($checkboxes !== FALSE) {
  display_order_by_combo_with_button($order_by);
  
  if ($photo_view == 'slide') {
    print emit_photos_per_page_combo('photos_per_page', $po_options['photos_per_page']);
  } else {
    print emit_photos_per_page_combo('rows_list_view', $po_options['rows_list_view']);
  }
 }

print "<table><tr><td>\n";

if ($checkboxes != FALSE) {
  display_list_slide_option($photo_view, "search.text.advanced.php?$search_string$arguments");
 }

if ($checkboxes > 0 && $po_user['id']) {
  theme_display_selectButton();
  print $thm_elem['button.add.to.spool'];
  print $thm_elem['button.bulk.update'];
 } 

print "</td></tr></table>\n";

if ($checkboxes > 0 && $po_user['id']) {
  $album_all = get_users_album($database);
  $folder_all = get_users_folder($database);
  print emit_folder_actions($folder_all, FALSE);
  if (count($album_all)) {
    print emit_album_actions($album_all, FALSE);
  }
}

if ($po_user['id']) {
  print emit_spool_actions($database, $po_user['id']);
  print emit_import_actions($database, $po_user['id']);
 }

print "</div>";

print "</form>";

print "&nbsp;<br/>\n";

site_footer($database);
site_epilog($database);
?>
