--  Copyright (c) 2007 Solomon Peachy (pizza@shaftnet.org)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.34.D to 2.35.A patch 
----------------------------------------------------------

-- Fix a bug in the count_photos_by_album() function.

create or replace function count_photos_by_album(integer, integer, varchar[]) returns bigint as 'select count(photo) from album_content where album = $1 and can_access_photo(photo, $2, $3); ' language 'sql';

--- Eliminate dead sequences and functions.

drop sequence submission_status_id_sequence;
drop sequence submission_id_sequence;
drop FUNCTION is_user_my_client(integer, integer);

-- De-Normalize the DB a little bit; get rid of crap:

alter table photo_tech add column scan_bitdepth2 integer;
update photo_tech set scan_bitdepth2 = (select d.value from
       scan_bitdepth d where d.identifier = scan_bitdepth)::integer where 
       scan_bitdepth is not null;
alter table photo_tech drop column scan_bitdepth;
alter table photo_tech rename column scan_bitdepth2 to scan_bitdepth;
drop table scan_bitdepth;
drop sequence scan_bitdepth_id_sequence;

alter table photo_tech add column scan_resolution2 integer;
update photo_tech set scan_resolution2 = (select d.value from
       scan_resolution d where d.identifier = scan_resolution)::integer where 
       scan_bitdepth is not null;
alter table photo_tech drop column scan_resolution;
alter table photo_tech rename column scan_resolution2 to scan_resolution;
drop table scan_resolution;
drop sequence scan_resolution_id_sequence;

drop table scan_multiscan cascade;
drop sequence scan_multiscan_id_sequence;

-- Eliminate the focal_length and aperture tables!

alter table photo_tech add column fl2 numeric;
update photo_tech set fl2 = (select d.value from
       focal_length d where d.identifier = focal_length) where 
       focal_length is not null;
alter table photo_tech drop column focal_length;
alter table photo_tech rename column fl2 to focal_length;

alter table photo_tech add column fl2 numeric;
update photo_tech set fl2 = (select d.value from
       aperture d where d.identifier = aperture) where 
       aperture is not null;
alter table photo_tech drop column aperture;
alter table photo_tech rename column fl2 to aperture;

drop view view_lens;
drop view view_lens_type;

alter table lens_type add column fl2 numeric;
update lens_type set fl2 = (select d.value from
       focal_length d where d.identifier = min_focal_length) where 
       min_focal_length is not null;
alter table lens_type drop column min_focal_length;
alter table lens_type rename column fl2 to min_focal_length;

alter table lens_type add column fl2 numeric;
update lens_type set fl2 = (select d.value from
       focal_length d where d.identifier = max_focal_length) where 
       max_focal_length is not null;
alter table lens_type drop column max_focal_length;
alter table lens_type rename column fl2 to max_focal_length;

alter table lens_type add column fl2 numeric;
update lens_type set fl2 = (select d.value from
       aperture d where d.identifier = min_aperture) where 
       min_aperture is not null;
alter table lens_type drop column min_aperture;
alter table lens_type rename column fl2 to min_aperture;

alter table lens_type add column fl2 numeric;
update lens_type set fl2 = (select d.value from
       aperture d where d.identifier = max_aperture) where 
       max_aperture is not null;
alter table lens_type drop column max_aperture;
alter table lens_type rename column fl2 to max_aperture;

create view view_lens as
 select lens.identifier, manufacturer.name, model, variation,
        min_aperture as s_min_aperture, 
	max_aperture as s_max_aperture, 
	min_focal_length as s_min_focal_length,
	max_focal_length as s_max_focal_length,
        lens.serial_number, lens.users,
        date_of_purchase, lens.purchased_new,
        manufacturer.url, (access_type.value) as access_rights,
	exiftool_lens_id
    from lens_type, lens, manufacturer, access_type
   where lens_type.manufacturer = manufacturer.identifier
     and lens_type.identifier = lens.type
     and lens.access_rights = access_type.identifier
   order by manufacturer.name, s_min_focal_length, model, variation;

create view view_lens_type as 
 select lens_type.identifier, manufacturer.name,
        model, variation, 
        min_aperture as s_min_aperture, 
	max_aperture as s_max_aperture, 
	min_focal_length as s_min_focal_length,
	max_focal_length as s_max_focal_length,
        manufacturer.url,
        exiftool_lens_id
    from lens_type, manufacturer
   where lens_type.manufacturer = manufacturer.identifier
   order by manufacturer.name, s_min_focal_length, model, variation;

drop table focal_length;
drop sequence focal_length_id_sequence;

drop table aperture;
drop sequence aperture_id_sequence;

-- Fix more stuff.

update photo_dupe set storage_location = '' where storage_location = 'null';

-- Update classic functions with ones that use proper permissions:

drop function count_photos_by_user(integer, integer);
drop function count_albums_by_user(integer, integer);
drop function count_folders_by_user(integer, integer);
drop function count_equipment_by_user(integer, integer);

CREATE OR REPLACE FUNCTION count_photos_by_user(integer, integer, varchar[]) returns bigint as ' select count(identifier) from photo where users = $1 and can_access_photo(identifier, $2, $3) ' language 'sql';

CREATE OR REPLACE FUNCTION count_folders_by_user(integer, integer, varchar[]) returns bigint as ' select count(identifier) from folder where users = $1 and can_access_folder(identifier, $2, $3) ' language 'sql';

CREATE OR REPLACE FUNCTION count_albums_by_user(integer, integer, varchar[]) returns bigint as ' select count(identifier) from album where users = $1 and can_access_album(identifier, $2, $3) ' language 'sql';

--- Username and password fields need to be expanded.
alter table users add col2 varchar(255);
update users set col2 = username;
alter table users drop column username;
alter table users rename column col2 to username;
alter table users alter column username set not null;

alter table users add col2 varchar(64);
update users set col2 = password;
alter table users drop column password;
alter table users rename column col2 to password;

update users set password = md5(password || username);

-- Duplicate Indices!

drop index photo_tech_id_idx;
drop index photo_version_photo;

-- Fix up some contraints!

alter table photo_version alter column original_image_name set not null;
alter table photo_version alter column date_of_creation set not null;
alter table photo_version alter column master set not null;

-- Some more triggers

CREATE OR REPLACE FUNCTION date_added_trig() RETURNS "trigger" AS '
BEGIN

IF TG_OP = ''INSERT'' THEN
   NEW.date_of_creation = now();
END IF;

RETURN NEW;

END;
' 
LANGUAGE 'plpgsql' VOLATILE;

CREATE TRIGGER photo_dupe_date_added_trigger
 BEFORE INSERT
    ON photo_dupe
   FOR EACH ROW EXECUTE PROCEDURE date_added_trig();

CREATE TRIGGER rating_date_added_trigger
 BEFORE INSERT
    ON rating
   FOR EACH ROW EXECUTE PROCEDURE date_added_trig();

CREATE TRIGGER client_date_added_trigger
 BEFORE INSERT
    ON client
   FOR EACH ROW EXECUTE PROCEDURE date_added_trig();

CREATE TRIGGER calendar_date_added_trigger
 BEFORE INSERT
    ON calendar
   FOR EACH ROW EXECUTE PROCEDURE date_added_trig();

--- To reduce round trips to the database..

CREATE TYPE folder_parents AS (identifier INTEGER, caption TEXT);

CREATE OR REPLACE FUNCTION get_folder_with_parents(int) RETURNS SETOF folder_parents AS '
DECLARE
  folder_id INT;
  row RECORD;
  retval RECORD;
BEGIN
  folder_id := $1;
  WHILE folder_id IS NOT NULL LOOP
    SELECT INTO row identifier, parent_folder, caption FROM folder WHERE identifier = folder_id LIMIT 1;
    SELECT INTO retval row.identifier, row.caption;
    RETURN NEXT retval;
    folder_id := row.parent_folder;
  END LOOP;

  RETURN;
END;
' 
LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION get_album_with_parents(int) RETURNS SETOF folder_parents AS '
DECLARE
  album_id INT;
  row RECORD;
  retval RECORD;
BEGIN
  album_id := $1;
  WHILE album_id IS NOT NULL LOOP
    SELECT INTO row identifier, parent_album, caption FROM album WHERE identifier = album_id LIMIT 1;
    SELECT INTO retval row.identifier, row.caption;
    RETURN NEXT retval;
    album_id := row.parent_album;
  END LOOP;

  RETURN;
END;
' 
LANGUAGE 'plpgsql';

-- Rearrange size codes
update files set size=0 where size = 3;

-- Fix up path substring!
update files set path = substring(path from 2) where path like '/%';

-- Add explicit backup lines to the files table.

alter table files add column sha1sum text;
alter table files add column backup_volume text;
create index files_path_index on files using btree("path");

-- Consolidate all contact info into the users table.
alter table users add column address1 varchar(100);
alter table users add column address2 varchar(100);
alter table users add column city varchar(100);
alter table users add column zipcode varchar(10);
alter table users add column state integer references state(identifier);
alter table users add column country integer references country(identifier);
alter table users add column phone varchar(40);
alter table users add column email varchar(150);
alter table users add column url varchar(150);

update users set address1=c.address1,
                 address2=c.address2,
                 city=c.city,
                 zipcode=c.zipcode,
                 state=c.state,
                 country=c.country,
                 phone=c.phone,
                 email=c.email,
                 url=c.url
  from contact_info c
where users.contact = c.identifier;

alter table users drop column contact;
drop view view_contact_info;
drop table contact_info;
drop sequence contact_info_id_sequence;

create view view_contact_info as
SELECT identifier, username, first_name, last_name,
       company, member_since, type, address1, address2, city, zipcode,
        ( SELECT state.value FROM state WHERE state.identifier = c.state) AS state,
        ( SELECT country.value FROM country WHERE country.identifier = c.country) AS country,
        c.phone, c.email, c.url
   FROM users c;

-- version update 
update version set identifier=3, description='2.35 customer release', 
 major=2, minor=35, variation='A';

